@@ -1,20 +1,22 @@
 <?php
-class DataProviderDebugTest extends PHPUnit_Framework_TestCase
-{
-    /**
-     * @dataProvider provider
-     */
-    public function testProvider()
-    {
-        $this->assertTrue(true);
-    }
+/*
+ * This file is part of PHPUnit.
+ *
+ * (c) Sebastian Bergmann <sebastian@phpunit.de>
+ *
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
+ */
+use PHPUnit\Framework\TestCase;
 
+class DataProviderDebugTest extends TestCase
+{
     public static function provider()
     {
         $obj2      = new \stdClass();
         $obj2->foo = 'bar';
 
-        $obj3 = (object) [1,2,"Test\r\n",4,5,6,7,8];
+        $obj3 = (object) [1, 2, "Test\r\n", 4, 5, 6, 7, 8];
 
         $obj = new \stdClass();
         //@codingStandardsIgnoreStart
@@ -32,17 +34,25 @@ class DataProviderDebugTest extends PHPUnit_Framework_TestCase
 
         $storage = new \SplObjectStorage();
         $storage->attach($obj2);
-        $storage->foo = $obj2;
+        $storage->foo = $obj;
 
         return [
             [null, true, 1, 1.0],
-            [1.2, fopen('php://memory', 'r'), '1'],
-            [[[1,2,3], [3,4,5]]],
+            [1.2, \fopen('php://memory', 'r'), '1'],
+            [[[1, 2, 3], [3, 4, 5]]],
             // \n\r and \r is converted to \n
             ["this\nis\na\nvery\nvery\nvery\nvery\nvery\nvery\rlong\n\rtext"],
             [new \stdClass(), $obj, [], $storage, $obj3],
-            [chr(0) . chr(1) . chr(2) . chr(3) . chr(4) . chr(5), implode('', array_map('chr', range(0x0e, 0x1f)))],
-            [chr(0x00) . chr(0x09)]
+            [\chr(0) . \chr(1) . \chr(2) . \chr(3) . \chr(4) . \chr(5), \implode('', \array_map('chr', \range(0x0e, 0x1f)))],
+            [\chr(0x00) . \chr(0x09)]
         ];
     }
+
+    /**
+     * @dataProvider provider
+     */
+    public function testProvider(): void
+    {
+        $this->assertTrue(true);
+    }
 }
