@@ -1,5 +1,12 @@
 <?php
-
+/*
+ * This file is part of PHPUnit.
+ *
+ * (c) Sebastian Bergmann <sebastian@phpunit.de>
+ *
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
+ */
 namespace My\Space;
 
 class ExceptionNamespaceTest extends \PHPUnit\Framework\TestCase
@@ -23,7 +30,7 @@ class ExceptionNamespaceTest extends \PHPUnit\Framework\TestCase
      * @expectedExceptionMessage My\Space\ExceptionNamespaceTest::ERROR_MESSAGE
      * @expectedExceptionCode My\Space\ExceptionNamespaceTest::ERROR_CODE
      */
-    public function testConstants()
+    public function testConstants(): void
     {
     }
 
@@ -32,7 +39,7 @@ class ExceptionNamespaceTest extends \PHPUnit\Framework\TestCase
      * @expectedExceptionCode My\Space\ExceptionNamespaceTest::UNKNOWN_CODE_CONSTANT
      * @expectedExceptionMessage My\Space\ExceptionNamespaceTest::UNKNOWN_MESSAGE_CONSTANT
      */
-    public function testUnknownConstants()
+    public function testUnknownConstants(): void
     {
     }
 }
